@@ -23,8 +23,7 @@
 #include <stdlib.h> // malloc, free
 #include <assert.h> // assert
 
-#include <libdivecomputer/cressi_leonardo.h>
-
+#include "cressi_leonardo.h"
 #include "context-private.h"
 #include "device-private.h"
 #include "serial.h"
@@ -50,7 +49,7 @@
 
 typedef struct cressi_leonardo_device_t {
 	dc_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
 	unsigned char fingerprint[5];
 } cressi_leonardo_device_t;
 
@@ -68,9 +67,13 @@ static const dc_device_vtable_t cressi_leonardo_device_vtable = {
 	NULL, /* write */
 	cressi_leonardo_device_dump, /* dump */
 	cressi_leonardo_device_foreach, /* foreach */
+	NULL, /* timesync */
 	cressi_leonardo_device_close /* close */
 };
 
+static dc_status_t
+cressi_leonardo_extract_dives (dc_device_t *abstract, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata);
+
 static void
 cressi_leonardo_make_ascii (const unsigned char raw[], unsigned int rsize, unsigned char ascii[], unsigned int asize)
 {
@@ -103,14 +106,14 @@ cressi_leonardo_packet (cressi_leonardo_device_t *device, const unsigned char co
 		return DC_STATUS_CANCELLED;
 
 	// Send the command to the device.
-	status = dc_serial_write (device->port, command, csize, NULL);
+	status = dc_iostream_write (device->iostream, command, csize, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
 	}
 
 	// Receive the answer of the device.
-	status = dc_serial_read (device->port, answer, asize, NULL);
+	status = dc_iostream_read (device->iostream, answer, asize, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -153,8 +156,8 @@ cressi_leonardo_transfer (cressi_leonardo_device_t *device, const unsigned char
 			return rc;
 
 		// Discard any garbage bytes.
-		dc_serial_sleep (device->port, 100);
-		dc_serial_purge (device->port, DC_DIRECTION_INPUT);
+		dc_iostream_sleep (device->iostream, 100);
+		dc_iostream_purge (device->iostream, DC_DIRECTION_INPUT);
 	}
 
 	return rc;
@@ -177,62 +180,62 @@ cressi_leonardo_device_open (dc_device_t **out, dc_context_t *context, const cha
 	}
 
 	// Set the default values.
-	device->port = NULL;
+	device->iostream = NULL;
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
 
 	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
+	status = dc_serial_open (&device->iostream, context, name);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to open the serial port.");
 		goto error_free;
 	}
 
 	// Set the serial communication protocol (115200 8N1).
-	status = dc_serial_configure (device->port, 115200, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
+	status = dc_iostream_configure (device->iostream, 115200, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_close;
 	}
 
 	// Set the timeout for receiving data (1000 ms).
-	status = dc_serial_set_timeout (device->port, 1000);
+	status = dc_iostream_set_timeout (device->iostream, 1000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
 		goto error_close;
 	}
 
 	// Set the RTS line.
-	status = dc_serial_set_rts (device->port, 1);
+	status = dc_iostream_set_rts (device->iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the RTS line.");
 		goto error_close;
 	}
 
 	// Set the DTR line.
-	status = dc_serial_set_dtr (device->port, 1);
+	status = dc_iostream_set_dtr (device->iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the DTR line.");
 		goto error_close;
 	}
 
-	dc_serial_sleep (device->port, 200);
+	dc_iostream_sleep (device->iostream, 200);
 
 	// Clear the DTR line.
-	status = dc_serial_set_dtr (device->port, 0);
+	status = dc_iostream_set_dtr (device->iostream, 0);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to clear the DTR line.");
 		goto error_close;
 	}
 
-	dc_serial_sleep (device->port, 100);
-	dc_serial_purge (device->port, DC_DIRECTION_ALL);
+	dc_iostream_sleep (device->iostream, 100);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	*out = (dc_device_t *) device;
 
 	return DC_STATUS_SUCCESS;
 
 error_close:
-	dc_serial_close (device->port);
+	dc_iostream_close (device->iostream);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
@@ -246,7 +249,7 @@ cressi_leonardo_device_close (dc_device_t *abstract)
 	dc_status_t rc = DC_STATUS_SUCCESS;
 
 	// Close the device.
-	rc = dc_serial_close (device->port);
+	rc = dc_iostream_close (device->iostream);
 	if (rc != DC_STATUS_SUCCESS) {
 		dc_status_set_error(&status, rc);
 	}
@@ -331,7 +334,7 @@ cressi_leonardo_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 	// Send the command header to the dive computer.
 	const unsigned char command[] = {0x7B, 0x31, 0x32, 0x33, 0x44, 0x42, 0x41, 0x7d};
-	status = dc_serial_write (device->port, command, sizeof (command), NULL);
+	status = dc_iostream_write (device->iostream, command, sizeof (command), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -339,7 +342,7 @@ cressi_leonardo_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 	// Receive the header packet.
 	unsigned char header[7] = {0};
-	status = dc_serial_read (device->port, header, sizeof (header), NULL);
+	status = dc_iostream_read (device->iostream, header, sizeof (header), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -361,7 +364,7 @@ cressi_leonardo_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 		// Increase the packet size if more data is immediately available.
 		size_t available = 0;
-		status = dc_serial_get_available (device->port, &available);
+		status = dc_iostream_get_available (device->iostream, &available);
 		if (status == DC_STATUS_SUCCESS && available > len)
 			len = available;
 
@@ -370,7 +373,7 @@ cressi_leonardo_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 			len = SZ_MEMORY - nbytes;
 
 		// Read the packet.
-		status = dc_serial_read (device->port, data + nbytes, len, NULL);
+		status = dc_iostream_read (device->iostream, data + nbytes, len, NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the answer.");
 			return status;
@@ -385,7 +388,7 @@ cressi_leonardo_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 	// Receive the trailer packet.
 	unsigned char trailer[4] = {0};
-	status = dc_serial_read (device->port, trailer, sizeof (trailer), NULL);
+	status = dc_iostream_read (device->iostream, trailer, sizeof (trailer), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -434,7 +437,7 @@ cressi_leonardo_device_foreach (dc_device_t *abstract, dc_dive_callback_t callba
 	return rc;
 }
 
-dc_status_t
+static dc_status_t
 cressi_leonardo_extract_dives (dc_device_t *abstract, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata)
 {
 	cressi_leonardo_device_t *device = (cressi_leonardo_device_t *) abstract;
@@ -446,32 +449,25 @@ cressi_leonardo_extract_dives (dc_device_t *abstract, const unsigned char data[]
 	if (size < SZ_MEMORY)
 		return DC_STATUS_DATAFORMAT;
 
-	// Locate the most recent dive.
-	// The device maintains an internal counter which is incremented for every
-	// dive, and the current value at the time of the dive is stored in the
-	// dive header. Thus the most recent dive will have the highest value.
-	unsigned int count = 0;
-	unsigned int latest = 0;
-	unsigned int maximum = 0;
-	for (unsigned int i = 0; i < RB_LOGBOOK_COUNT; ++i) {
-		unsigned int offset = RB_LOGBOOK_BEGIN + i * RB_LOGBOOK_SIZE;
+	// Get the number of dives.
+	//unsigned int ndives = array_uint16_le(data + 0x62);
 
-		// Ignore uninitialized header entries.
-		if (array_isequal (data + offset, RB_LOGBOOK_SIZE, 0xFF))
-			break;
+	// Get the logbook pointer.
+	unsigned int last = array_uint16_le(data + 0x64);
+	if (last < RB_LOGBOOK_BEGIN || last > RB_LOGBOOK_END ||
+		((last - RB_LOGBOOK_BEGIN) % RB_LOGBOOK_SIZE) != 0) {
+		ERROR (context, "Invalid logbook pointer (0x%04x).", last);
+		return DC_STATUS_DATAFORMAT;
+	}
 
-		// Get the internal dive number.
-		unsigned int current = array_uint16_le (data + offset);
-		if (current == 0xFFFF) {
-			WARNING (context, "Unexpected internal dive number found.");
-			break;
-		}
-		if (current > maximum) {
-			maximum = current;
-			latest = i;
-		}
+	// Convert to an index.
+	unsigned int latest = (last - RB_LOGBOOK_BEGIN) / RB_LOGBOOK_SIZE;
 
-		count++;
+	// Get the profile pointer.
+	unsigned int eop = array_uint16_le(data + 0x66);
+	if (eop < RB_PROFILE_BEGIN || last > RB_PROFILE_END) {
+		ERROR (context, "Invalid profile pointer (0x%04x).", eop);
+		return DC_STATUS_DATAFORMAT;
 	}
 
 	unsigned char *buffer = (unsigned char *) malloc (RB_LOGBOOK_SIZE + RB_PROFILE_END - RB_PROFILE_BEGIN);
@@ -480,12 +476,16 @@ cressi_leonardo_extract_dives (dc_device_t *abstract, const unsigned char data[]
 		return DC_STATUS_NOMEMORY;
 	}
 
-	unsigned int previous = 0;
+	unsigned int previous = eop;
 	unsigned int remaining = RB_PROFILE_END - RB_PROFILE_BEGIN;
-	for (unsigned int i = 0; i < count; ++i) {
+	for (unsigned int i = 0; i < RB_LOGBOOK_COUNT; ++i) {
 		unsigned int idx = (latest + RB_LOGBOOK_COUNT - i) % RB_LOGBOOK_COUNT;
 		unsigned int offset = RB_LOGBOOK_BEGIN + idx * RB_LOGBOOK_SIZE;
 
+		// Ignore uninitialized header entries.
+		if (array_isequal (data + offset, RB_LOGBOOK_SIZE, 0xFF))
+			break;
+
 		// Get the ringbuffer pointers.
 		unsigned int header = array_uint16_le (data + offset + 2);
 		unsigned int footer = array_uint16_le (data + offset + 4);
