@@ -22,11 +22,10 @@
 #include <string.h> // memcpy
 #include <stdlib.h> // malloc, free
 
-#include <libdivecomputer/oceanic_atom2.h>
-
+#include "oceanic_atom2.h"
+#include "oceanic_common.h"
 #include "context-private.h"
 #include "device-private.h"
-#include "oceanic_common.h"
 #include "serial.h"
 #include "array.h"
 #include "ringbuffer.h"
@@ -55,7 +54,7 @@
 
 typedef struct oceanic_atom2_device_t {
 	oceanic_common_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
 	unsigned int delay;
 	unsigned int bigpage;
 	unsigned char cache[256];
@@ -75,6 +74,7 @@ static const oceanic_common_device_vtable_t oceanic_atom2_device_vtable = {
 		oceanic_atom2_device_write, /* write */
 		oceanic_common_device_dump, /* dump */
 		oceanic_common_device_foreach, /* foreach */
+		NULL, /* timesync */
 		oceanic_atom2_device_close /* close */
 	},
 	oceanic_common_device_logbook,
@@ -102,13 +102,13 @@ static const oceanic_common_version_t oceanic_atom2_version[] = {
 
 static const oceanic_common_version_t oceanic_atom2a_version[] = {
 	{"MANTA  R\0\0  512K"},
-	{"WISDOM R\0\0  512K"},
 	{"INSIGHT2 \0\0 512K"},
 	{"OCEVEO30 \0\0 512K"},
 	{"ATMOSAI R\0\0 512K"},
 	{"PROPLUS2 \0\0 512K"},
 	{"OCEGEO20 \0\0 512K"},
 	{"OCE GEO R\0\0 512K"},
+	{"AQUAI200 \0\0 512K"},
 };
 
 static const oceanic_common_version_t oceanic_atom2b_version[] = {
@@ -133,6 +133,10 @@ static const oceanic_common_version_t oceanic_default_version[] = {
 	{"COMPMASK \0\0 512K"},
 };
 
+static const oceanic_common_version_t sherwood_wisdom_version[] = {
+	{"WISDOM R\0\0  512K"},
+};
+
 static const oceanic_common_version_t oceanic_proplus3_version[] = {
 	{"PROPLUS3 \0\0 512K"},
 };
@@ -289,6 +293,20 @@ static const oceanic_common_layout_t oceanic_atom2c_layout = {
 	0, /* pt_mode_serial */
 };
 
+static const oceanic_common_layout_t sherwood_wisdom_layout = {
+	0xFFF0, /* memsize */
+	0x0000, /* cf_devinfo */
+	0x0040, /* cf_pointers */
+	0x03D0, /* rb_logbook_begin */
+	0x0A40, /* rb_logbook_end */
+	8, /* rb_logbook_entry_size */
+	0x0A40, /* rb_profile_begin */
+	0xFE00, /* rb_profile_end */
+	0, /* pt_mode_global */
+	0, /* pt_mode_logbook */
+	0, /* pt_mode_serial */
+};
+
 static const oceanic_common_layout_t oceanic_proplus3_layout = {
 	0x10000, /* memsize */
 	0x0000, /* cf_devinfo */
@@ -453,11 +471,11 @@ oceanic_atom2_packet (oceanic_atom2_device_t *device, const unsigned char comman
 		return DC_STATUS_CANCELLED;
 
 	if (device->delay) {
-		dc_serial_sleep (device->port, device->delay);
+		dc_iostream_sleep (device->iostream, device->delay);
 	}
 
 	// Send the command to the dive computer.
-	status = dc_serial_write (device->port, command, csize, NULL);
+	status = dc_iostream_write (device->iostream, command, csize, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -471,7 +489,7 @@ oceanic_atom2_packet (oceanic_atom2_device_t *device, const unsigned char comman
 
 	// Receive the response (ACK/NAK) of the dive computer.
 	unsigned char response = 0;
-	status = dc_serial_read (device->port, &response, 1, NULL);
+	status = dc_iostream_read (device->iostream, &response, 1, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -485,7 +503,7 @@ oceanic_atom2_packet (oceanic_atom2_device_t *device, const unsigned char comman
 
 	if (asize) {
 		// Receive the answer of the dive computer.
-		status = dc_serial_read (device->port, answer, asize, NULL);
+		status = dc_iostream_read (device->iostream, answer, asize, NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the answer.");
 			return status;
@@ -534,8 +552,8 @@ oceanic_atom2_transfer (oceanic_atom2_device_t *device, const unsigned char comm
 			device->delay++;
 
 		// Delay the next attempt.
-		dc_serial_sleep (device->port, 100);
-		dc_serial_purge (device->port, DC_DIRECTION_INPUT);
+		dc_iostream_sleep (device->iostream, 100);
+		dc_iostream_purge (device->iostream, DC_DIRECTION_INPUT);
 	}
 
 	return DC_STATUS_SUCCESS;
@@ -556,14 +574,7 @@ oceanic_atom2_quit (oceanic_atom2_device_t *device)
 
 
 dc_status_t
-oceanic_atom2_device_open (dc_device_t **out, dc_context_t *context, const char *name)
-{
-	return oceanic_atom2_device_open2 (out, context, name, 0);
-}
-
-
-dc_status_t
-oceanic_atom2_device_open2 (dc_device_t **out, dc_context_t *context, const char *name, unsigned int model)
+oceanic_atom2_device_open (dc_device_t **out, dc_context_t *context, const char *name, unsigned int model)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	oceanic_atom2_device_t *device = NULL;
@@ -582,14 +593,14 @@ oceanic_atom2_device_open2 (dc_device_t **out, dc_context_t *context, const char
 	oceanic_common_device_init (&device->base);
 
 	// Set the default values.
-	device->port = NULL;
+	device->iostream = NULL;
 	device->delay = 0;
 	device->bigpage = 1; // no big pages
 	device->cached = INVALID;
 	memset(device->cache, 0, sizeof(device->cache));
 
 	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
+	status = dc_serial_open (&device->iostream, context, name);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to open the serial port.");
 		goto error_free;
@@ -602,28 +613,28 @@ oceanic_atom2_device_open2 (dc_device_t **out, dc_context_t *context, const char
 	}
 
 	// Set the serial communication protocol (38400 8N1).
-	status = dc_serial_configure (device->port, baudrate, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
+	status = dc_iostream_configure (device->iostream, baudrate, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_close;
 	}
 
 	// Set the timeout for receiving data (1000 ms).
-	status = dc_serial_set_timeout (device->port, 1000);
+	status = dc_iostream_set_timeout (device->iostream, 1000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
 		goto error_close;
 	}
 
 	// Give the interface 100 ms to settle and draw power up.
-	dc_serial_sleep (device->port, 100);
+	dc_iostream_sleep (device->iostream, 100);
 
 	// Set the DTR/RTS lines.
-	dc_serial_set_dtr(device->port, 1);
-	dc_serial_set_rts(device->port, 1);
+	dc_iostream_set_dtr(device->iostream, 1);
+	dc_iostream_set_rts(device->iostream, 1);
 
 	// Make sure everything is in a sane state.
-	dc_serial_purge (device->port, DC_DIRECTION_ALL);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	// Switch the device from surface mode into download mode. Before sending
 	// this command, the device needs to be in PC mode (automatically activated
@@ -653,6 +664,8 @@ oceanic_atom2_device_open2 (dc_device_t **out, dc_context_t *context, const char
 		device->base.layout = &oceanic_atom2b_layout;
 	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_atom2c_version)) {
 		device->base.layout = &oceanic_atom2c_layout;
+	} else if (OCEANIC_COMMON_MATCH (device->base.version, sherwood_wisdom_version)) {
+		device->base.layout = &sherwood_wisdom_layout;
 	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_proplus3_version)) {
 		device->base.layout = &oceanic_proplus3_layout;
 	} else if (OCEANIC_COMMON_MATCH (device->base.version, tusa_zenair_version)) {
@@ -699,7 +712,7 @@ oceanic_atom2_device_open2 (dc_device_t **out, dc_context_t *context, const char
 	return DC_STATUS_SUCCESS;
 
 error_close:
-	dc_serial_close (device->port);
+	dc_iostream_close (device->iostream);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
@@ -720,7 +733,7 @@ oceanic_atom2_device_close (dc_device_t *abstract)
 	}
 
 	// Close the device.
-	rc = dc_serial_close (device->port);
+	rc = dc_iostream_close (device->iostream);
 	if (rc != DC_STATUS_SUCCESS) {
 		dc_status_set_error(&status, rc);
 	}
