@@ -23,11 +23,10 @@
 #include <stdlib.h> // malloc, free
 #include <assert.h>
 
-#include <libdivecomputer/oceanic_vtpro.h>
-
+#include "oceanic_vtpro.h"
+#include "oceanic_common.h"
 #include "context-private.h"
 #include "device-private.h"
-#include "oceanic_common.h"
 #include "serial.h"
 #include "ringbuffer.h"
 #include "checksum.h"
@@ -51,7 +50,7 @@ typedef enum oceanic_vtpro_protocol_t {
 
 typedef struct oceanic_vtpro_device_t {
 	oceanic_common_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
 	unsigned int model;
 	oceanic_vtpro_protocol_t protocol;
 } oceanic_vtpro_device_t;
@@ -69,6 +68,7 @@ static const oceanic_common_device_vtable_t oceanic_vtpro_device_vtable = {
 		NULL, /* write */
 		oceanic_common_device_dump, /* dump */
 		oceanic_common_device_foreach, /* foreach */
+		NULL, /* timesync */
 		oceanic_vtpro_device_close /* close */
 	},
 	oceanic_vtpro_device_logbook,
@@ -140,7 +140,7 @@ oceanic_vtpro_send (oceanic_vtpro_device_t *device, const unsigned char command[
 		return DC_STATUS_CANCELLED;
 
 	// Send the command to the dive computer.
-	status = dc_serial_write (device->port, command, csize, NULL);
+	status = dc_iostream_write (device->iostream, command, csize, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -148,7 +148,7 @@ oceanic_vtpro_send (oceanic_vtpro_device_t *device, const unsigned char command[
 
 	// Receive the response (ACK/NAK) of the dive computer.
 	unsigned char response = NAK;
-	status = dc_serial_read (device->port, &response, 1, NULL);
+	status = dc_iostream_read (device->iostream, &response, 1, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -189,7 +189,7 @@ oceanic_vtpro_transfer (oceanic_vtpro_device_t *device, const unsigned char comm
 
 	if (asize) {
 		// Receive the answer of the dive computer.
-		status = dc_serial_read (device->port, answer, asize, NULL);
+		status = dc_iostream_read (device->iostream, answer, asize, NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the answer.");
 			return status;
@@ -210,7 +210,7 @@ oceanic_vtpro_init (oceanic_vtpro_device_t *device)
 	unsigned char command[2][2] = {
 		{0xAA, 0x00},
 		{0x20, 0x00}};
-	status = dc_serial_write (device->port, command[device->protocol], sizeof (command[device->protocol]), NULL);
+	status = dc_iostream_write (device->iostream, command[device->protocol], sizeof (command[device->protocol]), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -218,7 +218,7 @@ oceanic_vtpro_init (oceanic_vtpro_device_t *device)
 
 	// Receive the answer of the dive computer.
 	unsigned char answer[13] = {0};
-	status = dc_serial_read (device->port, answer, sizeof (answer), NULL);
+	status = dc_iostream_read (device->iostream, answer, sizeof (answer), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -269,9 +269,9 @@ oceanic_vtpro_calibrate (oceanic_vtpro_device_t *device)
 	// device needs approximately 6 seconds to respond.
 	unsigned char answer[2] = {0};
 	unsigned char command[2] = {0x18, 0x00};
-	dc_serial_set_timeout (device->port, 9000);
+	dc_iostream_set_timeout (device->iostream, 9000);
 	dc_status_t rc = oceanic_vtpro_transfer (device, command, sizeof (command), answer, sizeof (answer));
-	dc_serial_set_timeout (device->port, 3000);
+	dc_iostream_set_timeout (device->iostream, 3000);
 	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
@@ -337,7 +337,7 @@ oceanic_aeris500ai_device_logbook (dc_device_t *abstract, dc_event_progress_t *p
 	for (unsigned int i = 0; i < last + 1; ++i) {
 		// Receive the answer of the dive computer.
 		unsigned char answer[PAGESIZE / 2 + 1] = {0};
-		rc = dc_serial_read (device->port, answer, sizeof(answer), NULL);
+		rc = dc_iostream_read (device->iostream, answer, sizeof(answer), NULL);
 		if (rc != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the answer.");
 			return rc;
@@ -385,14 +385,7 @@ oceanic_vtpro_device_logbook (dc_device_t *abstract, dc_event_progress_t *progre
 }
 
 dc_status_t
-oceanic_vtpro_device_open (dc_device_t **out, dc_context_t *context, const char *name)
-{
-	return oceanic_vtpro_device_open2 (out, context, name, 0);
-}
-
-
-dc_status_t
-oceanic_vtpro_device_open2 (dc_device_t **out, dc_context_t *context, const char *name, unsigned int model)
+oceanic_vtpro_device_open (dc_device_t **out, dc_context_t *context, const char *name, unsigned int model)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	oceanic_vtpro_device_t *device = NULL;
@@ -414,7 +407,7 @@ oceanic_vtpro_device_open2 (dc_device_t **out, dc_context_t *context, const char
 	device->base.multipage = MULTIPAGE;
 
 	// Set the default values.
-	device->port = NULL;
+	device->iostream = NULL;
 	device->model = model;
 	if (model == AERIS500AI) {
 		device->protocol = INTR;
@@ -423,45 +416,45 @@ oceanic_vtpro_device_open2 (dc_device_t **out, dc_context_t *context, const char
 	}
 
 	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
+	status = dc_serial_open (&device->iostream, context, name);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to open the serial port.");
 		goto error_free;
 	}
 
 	// Set the serial communication protocol (9600 8N1).
-	status = dc_serial_configure (device->port, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
+	status = dc_iostream_configure (device->iostream, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_close;
 	}
 
 	// Set the timeout for receiving data (3000 ms).
-	status = dc_serial_set_timeout (device->port, 3000);
+	status = dc_iostream_set_timeout (device->iostream, 3000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
 		goto error_close;
 	}
 
 	// Set the DTR line.
-	status = dc_serial_set_dtr (device->port, 1);
+	status = dc_iostream_set_dtr (device->iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the DTR line.");
 		goto error_close;
 	}
 
 	// Set the RTS line.
-	status = dc_serial_set_rts (device->port, 1);
+	status = dc_iostream_set_rts (device->iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the RTS line.");
 		goto error_close;
 	}
 
 	// Give the interface 100 ms to settle and draw power up.
-	dc_serial_sleep (device->port, device->protocol == MOD ? 100 : 1000);
+	dc_iostream_sleep (device->iostream, device->protocol == MOD ? 100 : 1000);
 
 	// Make sure everything is in a sane state.
-	dc_serial_purge (device->port, DC_DIRECTION_ALL);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	// Initialize the data cable (MOD mode).
 	status = oceanic_vtpro_init (device);
@@ -502,7 +495,7 @@ oceanic_vtpro_device_open2 (dc_device_t **out, dc_context_t *context, const char
 	return DC_STATUS_SUCCESS;
 
 error_close:
-	dc_serial_close (device->port);
+	dc_iostream_close (device->iostream);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
@@ -523,7 +516,7 @@ oceanic_vtpro_device_close (dc_device_t *abstract)
 	}
 
 	// Close the device.
-	rc = dc_serial_close (device->port);
+	rc = dc_iostream_close (device->iostream);
 	if (rc != DC_STATUS_SUCCESS) {
 		dc_status_set_error(&status, rc);
 	}
