@@ -22,8 +22,7 @@
 #include <string.h> // memcmp, memcpy
 #include <stdlib.h> // malloc, free
 
-#include <libdivecomputer/reefnet_sensuspro.h>
-
+#include "reefnet_sensuspro.h"
 #include "context-private.h"
 #include "device-private.h"
 #include "serial.h"
@@ -37,7 +36,7 @@
 
 typedef struct reefnet_sensuspro_device_t {
 	dc_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
 	unsigned char handshake[SZ_HANDSHAKE];
 	unsigned int timestamp;
 	unsigned int devtime;
@@ -57,9 +56,12 @@ static const dc_device_vtable_t reefnet_sensuspro_device_vtable = {
 	NULL, /* write */
 	reefnet_sensuspro_device_dump, /* dump */
 	reefnet_sensuspro_device_foreach, /* foreach */
+	NULL, /* timesync */
 	reefnet_sensuspro_device_close /* close */
 };
 
+static dc_status_t
+reefnet_sensuspro_extract_dives (dc_device_t *device, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata);
 
 dc_status_t
 reefnet_sensuspro_device_open (dc_device_t **out, dc_context_t *context, const char *name)
@@ -78,42 +80,42 @@ reefnet_sensuspro_device_open (dc_device_t **out, dc_context_t *context, const c
 	}
 
 	// Set the default values.
-	device->port = NULL;
+	device->iostream = NULL;
 	device->timestamp = 0;
 	device->systime = (dc_ticks_t) -1;
 	device->devtime = 0;
 	memset (device->handshake, 0, sizeof (device->handshake));
 
 	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
+	status = dc_serial_open (&device->iostream, context, name);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to open the serial port.");
 		goto error_free;
 	}
 
 	// Set the serial communication protocol (19200 8N1).
-	status = dc_serial_configure (device->port, 19200, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
+	status = dc_iostream_configure (device->iostream, 19200, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_close;
 	}
 
 	// Set the timeout for receiving data (3000ms).
-	status = dc_serial_set_timeout (device->port, 3000);
+	status = dc_iostream_set_timeout (device->iostream, 3000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
 		goto error_close;
 	}
 
 	// Make sure everything is in a sane state.
-	dc_serial_purge (device->port, DC_DIRECTION_ALL);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	*out = (dc_device_t*) device;
 
 	return DC_STATUS_SUCCESS;
 
 error_close:
-	dc_serial_close (device->port);
+	dc_iostream_close (device->iostream);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
@@ -128,7 +130,7 @@ reefnet_sensuspro_device_close (dc_device_t *abstract)
 	dc_status_t rc = DC_STATUS_SUCCESS;
 
 	// Close the device.
-	rc = dc_serial_close (device->port);
+	rc = dc_iostream_close (device->iostream);
 	if (rc != DC_STATUS_SUCCESS) {
 		dc_status_set_error(&status, rc);
 	}
@@ -180,18 +182,18 @@ reefnet_sensuspro_handshake (reefnet_sensuspro_device_t *device)
 	dc_device_t *abstract = (dc_device_t *) device;
 
 	// Assert a break condition.
-	dc_serial_set_break (device->port, 1);
+	dc_iostream_set_break (device->iostream, 1);
 
 	// Receive the handshake from the dive computer.
 	unsigned char handshake[SZ_HANDSHAKE + 2] = {0};
-	status = dc_serial_read (device->port, handshake, sizeof (handshake), NULL);
+	status = dc_iostream_read (device->iostream, handshake, sizeof (handshake), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the handshake.");
 		return status;
 	}
 
 	// Clear the break condition again.
-	dc_serial_set_break (device->port, 0);
+	dc_iostream_set_break (device->iostream, 0);
 
 	// Verify the checksum of the handshake packet.
 	unsigned short crc = array_uint16_le (handshake + SZ_HANDSHAKE);
@@ -227,7 +229,7 @@ reefnet_sensuspro_handshake (reefnet_sensuspro_device_t *device)
 	vendor.size = sizeof (device->handshake);
 	device_event_emit (abstract, DC_EVENT_VENDOR, &vendor);
 
-	dc_serial_sleep (device->port, 10);
+	dc_iostream_sleep (device->iostream, 10);
 
 	return DC_STATUS_SUCCESS;
 }
@@ -245,7 +247,7 @@ reefnet_sensuspro_send (reefnet_sensuspro_device_t *device, unsigned char comman
 		return rc;
 
 	// Send the instruction code to the device.
-	status = dc_serial_write (device->port, &command, 1, NULL);
+	status = dc_iostream_write (device->iostream, &command, 1, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -285,7 +287,7 @@ reefnet_sensuspro_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 		if (len > 256)
 			len = 256;
 
-		status = dc_serial_read (device->port, answer + nbytes, len, NULL);
+		status = dc_iostream_read (device->iostream, answer + nbytes, len, NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the answer.");
 			return status;
@@ -350,9 +352,9 @@ reefnet_sensuspro_device_write_interval (dc_device_t *abstract, unsigned char in
 	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
-	dc_serial_sleep (device->port, 10);
+	dc_iostream_sleep (device->iostream, 10);
 
-	status = dc_serial_write (device->port, &interval, 1, NULL);
+	status = dc_iostream_write (device->iostream, &interval, 1, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the data packet.");
 		return status;
@@ -362,7 +364,7 @@ reefnet_sensuspro_device_write_interval (dc_device_t *abstract, unsigned char in
 }
 
 
-dc_status_t
+static dc_status_t
 reefnet_sensuspro_extract_dives (dc_device_t *abstract, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata)
 {
 	reefnet_sensuspro_device_t *device = (reefnet_sensuspro_device_t*) abstract;
