@@ -25,12 +25,29 @@
 #include <windows.h>
 
 #include "serial.h"
+
 #include "common-private.h"
 #include "context-private.h"
+#include "iostream-private.h"
 
-struct dc_serial_t {
-	/* Library context. */
-	dc_context_t *context;
+static dc_status_t dc_serial_set_timeout (dc_iostream_t *iostream, int timeout);
+static dc_status_t dc_serial_set_latency (dc_iostream_t *iostream, unsigned int value);
+static dc_status_t dc_serial_set_halfduplex (dc_iostream_t *iostream, unsigned int value);
+static dc_status_t dc_serial_set_break (dc_iostream_t *iostream, unsigned int value);
+static dc_status_t dc_serial_set_dtr (dc_iostream_t *iostream, unsigned int value);
+static dc_status_t dc_serial_set_rts (dc_iostream_t *iostream, unsigned int value);
+static dc_status_t dc_serial_get_lines (dc_iostream_t *iostream, unsigned int *value);
+static dc_status_t dc_serial_get_available (dc_iostream_t *iostream, size_t *value);
+static dc_status_t dc_serial_configure (dc_iostream_t *iostream, unsigned int baudrate, unsigned int databits, dc_parity_t parity, dc_stopbits_t stopbits, dc_flowcontrol_t flowcontrol);
+static dc_status_t dc_serial_read (dc_iostream_t *iostream, void *data, size_t size, size_t *actual);
+static dc_status_t dc_serial_write (dc_iostream_t *iostream, const void *data, size_t size, size_t *actual);
+static dc_status_t dc_serial_flush (dc_iostream_t *iostream);
+static dc_status_t dc_serial_purge (dc_iostream_t *iostream, dc_direction_t direction);
+static dc_status_t dc_serial_sleep (dc_iostream_t *iostream, unsigned int milliseconds);
+static dc_status_t dc_serial_close (dc_iostream_t *iostream);
+
+typedef struct dc_serial_t {
+	dc_iostream_t base;
 	/*
 	 * The file descriptor corresponding to the serial port.
 	 */
@@ -46,6 +63,25 @@ struct dc_serial_t {
 	int halfduplex;
 	unsigned int baudrate;
 	unsigned int nbits;
+} dc_serial_t;
+
+static const dc_iostream_vtable_t dc_serial_vtable = {
+	sizeof(dc_serial_t),
+	dc_serial_set_timeout, /* set_timeout */
+	dc_serial_set_latency, /* set_latency */
+	dc_serial_set_halfduplex, /* set_halfduplex */
+	dc_serial_set_break, /* set_break */
+	dc_serial_set_dtr, /* set_dtr */
+	dc_serial_set_rts, /* set_rts */
+	dc_serial_get_lines, /* get_lines */
+	dc_serial_get_available, /* get_received */
+	dc_serial_configure, /* configure */
+	dc_serial_read, /* read */
+	dc_serial_write, /* write */
+	dc_serial_flush, /* flush */
+	dc_serial_purge, /* purge */
+	dc_serial_sleep, /* sleep */
+	dc_serial_close, /* close */
 };
 
 static dc_status_t
@@ -120,19 +156,24 @@ dc_serial_enumerate (dc_serial_callback_t callback, void *userdata)
 }
 
 dc_status_t
-dc_serial_open (dc_serial_t **out, dc_context_t *context, const char *name)
+dc_serial_open (dc_iostream_t **out, dc_context_t *context, const char *name)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_serial_t *device = NULL;
 
-	if (out == NULL)
+	if (out == NULL || name == NULL)
 		return DC_STATUS_INVALIDARGS;
 
-	INFO (context, "Open: name=%s", name ? name : "");
+	// Are we using custom IO?
+	if (_dc_context_custom_io(context))
+		return dc_custom_io_serial_open(out, context, name);
+
+	INFO (context, "Open: name=%s", name);
 
 	// Build the device name.
 	const char *devname = NULL;
 	char buffer[MAX_PATH] = "\\\\.\\";
-	if (name && strncmp (name, buffer, 4) != 0) {
+	if (strncmp (name, buffer, 4) != 0) {
 		size_t length = strlen (name) + 1;
 		if (length + 4 > sizeof (buffer))
 			return DC_STATUS_NOMEMORY;
@@ -143,22 +184,17 @@ dc_serial_open (dc_serial_t **out, dc_context_t *context, const char *name)
 	}
 
 	// Allocate memory.
-	dc_serial_t *device = (dc_serial_t *) malloc (sizeof (dc_serial_t));
+	device = (dc_serial_t *) dc_iostream_allocate (context, &dc_serial_vtable);
 	if (device == NULL) {
 		SYSERROR (context, ERROR_OUTOFMEMORY);
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Library context.
-	device->context = context;
-
 	// Default to full-duplex.
 	device->halfduplex = 0;
 	device->baudrate = 0;
 	device->nbits = 0;
 
-	RETURN_IF_CUSTOM_SERIAL(context, *out = device, open, context, name);
-
 	// Open the device.
 	device->hFile = CreateFileA (devname,
 			GENERIC_READ | GENERIC_WRITE, 0,
@@ -185,64 +221,51 @@ dc_serial_open (dc_serial_t **out, dc_context_t *context, const char *name)
 		goto error_close;
 	}
 
-	*out = device;
+	*out = (dc_iostream_t *) device;
 
 	return DC_STATUS_SUCCESS;
 
 error_close:
 	CloseHandle (device->hFile);
 error_free:
-	free (device);
+	dc_iostream_deallocate ((dc_iostream_t *) device);
 	return status;
 }
 
-dc_status_t
-dc_serial_close (dc_serial_t *device)
+static dc_status_t
+dc_serial_close (dc_iostream_t *abstract)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
-
-	if (device == NULL)
-		return DC_STATUS_SUCCESS;
-
-	RETURN_IF_CUSTOM_SERIAL(device->context, free(device), close);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	// Restore the initial communication settings and timeouts.
 	if (!SetCommState (device->hFile, &device->dcb) ||
 		!SetCommTimeouts (device->hFile, &device->timeouts)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		dc_status_set_error(&status, syserror (errcode));
 	}
 
 	// Close the device.
 	if (!CloseHandle (device->hFile)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		dc_status_set_error(&status, syserror (errcode));
 	}
 
-	// Free memory.
-	free (device);
-
 	return status;
 }
 
-dc_status_t
-dc_serial_configure (dc_serial_t *device, unsigned int baudrate, unsigned int databits, dc_parity_t parity, dc_stopbits_t stopbits, dc_flowcontrol_t flowcontrol)
+static dc_status_t
+dc_serial_configure (dc_iostream_t *abstract, unsigned int baudrate, unsigned int databits, dc_parity_t parity, dc_stopbits_t stopbits, dc_flowcontrol_t flowcontrol)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "Configure: baudrate=%i, databits=%i, parity=%i, stopbits=%i, flowcontrol=%i",
-		baudrate, databits, parity, stopbits, flowcontrol);
-
-	RETURN_IF_CUSTOM_SERIAL(device->context, , configure, baudrate, databits, parity, stopbits, flowcontrol);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	// Retrieve the current settings.
 	DCB dcb;
 	if (!GetCommState (device->hFile, &dcb)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
@@ -332,7 +355,7 @@ dc_serial_configure (dc_serial_t *device, unsigned int baudrate, unsigned int da
 	// Apply the new settings.
 	if (!SetCommState (device->hFile, &dcb)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
@@ -342,21 +365,16 @@ dc_serial_configure (dc_serial_t *device, unsigned int baudrate, unsigned int da
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_set_timeout (dc_serial_t *device, int timeout)
+static dc_status_t
+dc_serial_set_timeout (dc_iostream_t *abstract, int timeout)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "Timeout: value=%i", timeout);
-
-	RETURN_IF_CUSTOM_SERIAL(device->context, , set_timeout, timeout);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	// Retrieve the current timeouts.
 	COMMTIMEOUTS timeouts;
 	if (!GetCommTimeouts (device->hFile, &timeouts)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
@@ -387,58 +405,39 @@ dc_serial_set_timeout (dc_serial_t *device, int timeout)
 	// Activate the new timeouts.
 	if (!SetCommTimeouts (device->hFile, &timeouts)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_set_halfduplex (dc_serial_t *device, unsigned int value)
+static dc_status_t
+dc_serial_set_halfduplex (dc_iostream_t *abstract, unsigned int value)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	RETURN_IF_CUSTOM_SERIAL(device->context, , set_halfduplex, value);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	device->halfduplex = value;
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_set_latency (dc_serial_t *device, unsigned int value)
+static dc_status_t
+dc_serial_set_latency (dc_iostream_t *abstract, unsigned int value)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_read (dc_serial_t *device, void *data, size_t size, size_t *actual)
+static dc_status_t
+dc_serial_read (dc_iostream_t *abstract, void *data, size_t size, size_t *actual)
 {
-	size_t nbytes = 0;
 	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_serial_t *device = (dc_serial_t *) abstract;
 	DWORD dwRead = 0;
 
-	if (device == NULL) {
-		status = DC_STATUS_INVALIDARGS;
-		goto out_invalidargs;
-	}
-
-	RETURN_IF_CUSTOM_SERIAL(device->context,
-			{
-				HEXDUMP (device->context, DC_LOGLEVEL_INFO, "Custom Read", (unsigned char *) data, nbytes);
-				if (actual)
-					*actual = nbytes;
-			},
-			read, data, size, &nbytes);
-
 	if (!ReadFile (device->hFile, data, size, &dwRead, NULL)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		status = syserror (errcode);
 		goto out;
 	}
@@ -448,42 +447,26 @@ dc_serial_read (dc_serial_t *device, void *data, size_t size, size_t *actual)
 	}
 
 out:
-	HEXDUMP (device->context, DC_LOGLEVEL_INFO, "Read", (unsigned char *) data, dwRead);
-
-out_invalidargs:
 	if (actual)
 		*actual = dwRead;
 
 	return status;
 }
 
-dc_status_t
-dc_serial_write (dc_serial_t *device, const void *data, size_t size, size_t *actual)
+static dc_status_t
+dc_serial_write (dc_iostream_t *abstract, const void *data, size_t size, size_t *actual)
 {
-	size_t nbytes = 0;
 	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_serial_t *device = (dc_serial_t *) abstract;
 	DWORD dwWritten = 0;
 
-	if (device == NULL) {
-		status = DC_STATUS_INVALIDARGS;
-		goto out_invalidargs;
-	}
-
-	RETURN_IF_CUSTOM_SERIAL(device->context,
-			{
-				HEXDUMP (device->context, DC_LOGLEVEL_INFO, "Custom Write", (unsigned char *) data, nbytes);
-				if (actual)
-					*actual = nbytes;
-			},
-			write, data, size, &nbytes);
-
 	LARGE_INTEGER begin, end, freq;
 	if (device->halfduplex) {
 		// Get the current time.
 		if (!QueryPerformanceFrequency(&freq) ||
 			!QueryPerformanceCounter(&begin)) {
 			DWORD errcode = GetLastError ();
-			SYSERROR (device->context, errcode);
+			SYSERROR (abstract->context, errcode);
 			status = syserror (errcode);
 			goto out;
 		}
@@ -491,7 +474,7 @@ dc_serial_write (dc_serial_t *device, const void *data, size_t size, size_t *act
 
 	if (!WriteFile (device->hFile, data, size, &dwWritten, NULL)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		status = syserror (errcode);
 		goto out;
 	}
@@ -500,7 +483,7 @@ dc_serial_write (dc_serial_t *device, const void *data, size_t size, size_t *act
 		// Get the current time.
 		if (!QueryPerformanceCounter(&end))  {
 			DWORD errcode = GetLastError ();
-			SYSERROR (device->context, errcode);
+			SYSERROR (abstract->context, errcode);
 			status = syserror (errcode);
 			goto out;
 		}
@@ -519,7 +502,7 @@ dc_serial_write (dc_serial_t *device, const void *data, size_t size, size_t *act
 			// The remaining time is rounded up to the nearest millisecond
 			// because the Windows Sleep() function doesn't have a higher
 			// resolution.
-			dc_serial_sleep (device, (remaining + 999) / 1000);
+			dc_serial_sleep (abstract, (remaining + 999) / 1000);
 		}
 	}
 
@@ -528,24 +511,16 @@ dc_serial_write (dc_serial_t *device, const void *data, size_t size, size_t *act
 	}
 
 out:
-	HEXDUMP (device->context, DC_LOGLEVEL_INFO, "Write", (unsigned char *) data, dwWritten);
-
-out_invalidargs:
 	if (actual)
 		*actual = dwWritten;
 
 	return status;
 }
 
-dc_status_t
-dc_serial_purge (dc_serial_t *device, dc_direction_t direction)
+static dc_status_t
+dc_serial_purge (dc_iostream_t *abstract, dc_direction_t direction)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "Purge: direction=%u", direction);
-
-	RETURN_IF_CUSTOM_SERIAL(device->context, , purge, direction);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	DWORD flags = 0;
 
@@ -565,50 +540,42 @@ dc_serial_purge (dc_serial_t *device, dc_direction_t direction)
 
 	if (!PurgeComm (device->hFile, flags)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_flush (dc_serial_t *device)
+static dc_status_t
+dc_serial_flush (dc_iostream_t *abstract)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "Flush: none");
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	if (!FlushFileBuffers (device->hFile)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_set_break (dc_serial_t *device, unsigned int level)
+static dc_status_t
+dc_serial_set_break (dc_iostream_t *abstract, unsigned int level)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "Break: value=%i", level);
-
-	RETURN_IF_CUSTOM_SERIAL(device->context, , set_break, level);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	if (level) {
 		if (!SetCommBreak (device->hFile)) {
 			DWORD errcode = GetLastError ();
-			SYSERROR (device->context, errcode);
+			SYSERROR (abstract->context, errcode);
 			return syserror (errcode);
 		}
 	} else {
 		if (!ClearCommBreak (device->hFile)) {
 			DWORD errcode = GetLastError ();
-			SYSERROR (device->context, errcode);
+			SYSERROR (abstract->context, errcode);
 			return syserror (errcode);
 		}
 	}
@@ -616,61 +583,48 @@ dc_serial_set_break (dc_serial_t *device, unsigned int level)
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_set_dtr (dc_serial_t *device, unsigned int level)
+static dc_status_t
+dc_serial_set_dtr (dc_iostream_t *abstract, unsigned int level)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "DTR: value=%i", level);
-
-	RETURN_IF_CUSTOM_SERIAL(device->context, , set_dtr, level);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	int status = (level ? SETDTR : CLRDTR);
 
 	if (!EscapeCommFunction (device->hFile, status)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_set_rts (dc_serial_t *device, unsigned int level)
+static dc_status_t
+dc_serial_set_rts (dc_iostream_t *abstract, unsigned int level)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "RTS: value=%i", level);
-
-	RETURN_IF_CUSTOM_SERIAL(device->context, , set_rts, level);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	int status = (level ? SETRTS : CLRRTS);
 
 	if (!EscapeCommFunction (device->hFile, status)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_get_available (dc_serial_t *device, size_t *value)
+static dc_status_t
+dc_serial_get_available (dc_iostream_t *abstract, size_t *value)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	RETURN_IF_CUSTOM_SERIAL(device->context, , get_available, value);
+	dc_serial_t *device = (dc_serial_t *) abstract;
 
 	COMSTAT stats;
 
 	if (!ClearCommError (device->hFile, NULL, &stats)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
@@ -680,18 +634,16 @@ dc_serial_get_available (dc_serial_t *device, size_t *value)
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_get_lines (dc_serial_t *device, unsigned int *value)
+static dc_status_t
+dc_serial_get_lines (dc_iostream_t *abstract, unsigned int *value)
 {
+	dc_serial_t *device = (dc_serial_t *) abstract;
 	unsigned int lines = 0;
 
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
 	DWORD stats = 0;
 	if (!GetCommModemStatus (device->hFile, &stats)) {
 		DWORD errcode = GetLastError ();
-		SYSERROR (device->context, errcode);
+		SYSERROR (abstract->context, errcode);
 		return syserror (errcode);
 	}
 
@@ -710,14 +662,9 @@ dc_serial_get_lines (dc_serial_t *device, unsigned int *value)
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_serial_sleep (dc_serial_t *device, unsigned int timeout)
+static dc_status_t
+dc_serial_sleep (dc_iostream_t *abstract, unsigned int timeout)
 {
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	INFO (device->context, "Sleep: value=%u", timeout);
-
 	Sleep (timeout);
 
 	return DC_STATUS_SUCCESS;
