@@ -41,21 +41,21 @@ shearwater_common_open (shearwater_common_device_t *device, dc_context_t *contex
 	dc_status_t status = DC_STATUS_SUCCESS;
 
 	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
+	status = dc_serial_open (&device->iostream, context, name);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to open the serial port.");
 		return status;
 	}
 
 	// Set the serial communication protocol (115200 8N1).
-	status = dc_serial_configure (device->port, 115200, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
+	status = dc_iostream_configure (device->iostream, 115200, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_close;
 	}
 
 	// Set the timeout for receiving data (3000ms).
-	status = dc_serial_set_timeout (device->port, 3000);
+	status = dc_iostream_set_timeout (device->iostream, 3000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
 		status = DC_STATUS_IO;
@@ -63,13 +63,13 @@ shearwater_common_open (shearwater_common_device_t *device, dc_context_t *contex
 	}
 
 	// Make sure everything is in a sane state.
-	dc_serial_sleep (device->port, 300);
-	dc_serial_purge (device->port, DC_DIRECTION_ALL);
+	dc_iostream_sleep (device->iostream, 300);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	return DC_STATUS_SUCCESS;
 
 error_close:
-	dc_serial_close (device->port);
+	dc_iostream_close (device->iostream);
 	return status;
 }
 
@@ -78,7 +78,7 @@ dc_status_t
 shearwater_common_close (shearwater_common_device_t *device)
 {
 	// Close the device.
-	return dc_serial_close (device->port);
+	return dc_iostream_close (device->iostream);
 }
 
 
@@ -154,7 +154,7 @@ shearwater_common_slip_write (shearwater_common_device_t *device, const unsigned
 #if 0
 	// Send an initial END character to flush out any data that may have
 	// accumulated in the receiver due to line noise.
-	status = dc_serial_write (device->port, end, sizeof (end), NULL);
+	status = dc_iostream_write (device->iostream, end, sizeof (end), NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		return status;
 	}
@@ -183,7 +183,7 @@ shearwater_common_slip_write (shearwater_common_device_t *device, const unsigned
 
 		// Flush the buffer if necessary.
 		if (nbytes + len + sizeof(end) > sizeof(buffer)) {
-			status = dc_serial_write (device->port, buffer, nbytes, NULL);
+			status = dc_iostream_write (device->iostream, buffer, nbytes, NULL);
 			if (status != DC_STATUS_SUCCESS) {
 				return status;
 			}
@@ -201,7 +201,7 @@ shearwater_common_slip_write (shearwater_common_device_t *device, const unsigned
 	nbytes += sizeof(end);
 
 	// Flush the buffer.
-	status = dc_serial_write (device->port, buffer, nbytes, NULL);
+	status = dc_iostream_write (device->iostream, buffer, nbytes, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		return status;
 	}
@@ -224,7 +224,7 @@ shearwater_common_slip_read (shearwater_common_device_t *device, unsigned char d
 		unsigned char c = 0;
 
 		// Get a single character to process.
-		status = dc_serial_read (device->port, &c, 1, NULL);
+		status = dc_iostream_read (device->iostream, &c, 1, NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			return status;
 		}
@@ -243,7 +243,7 @@ shearwater_common_slip_read (shearwater_common_device_t *device, unsigned char d
 		case ESC:
 			// If it's an ESC character, get another character and then
 			// figure out what to store in the packet based on that.
-			status = dc_serial_read (device->port, &c, 1, NULL);
+			status = dc_iostream_read (device->iostream, &c, 1, NULL);
 			if (status != DC_STATUS_SUCCESS) {
 				return status;
 			}
@@ -343,7 +343,7 @@ shearwater_common_transfer (shearwater_common_device_t *device, const unsigned c
 
 
 dc_status_t
-shearwater_common_download (shearwater_common_device_t *device, dc_buffer_t *buffer, unsigned int address, unsigned int size, unsigned int compression)
+shearwater_common_download (shearwater_common_device_t *device, dc_buffer_t *buffer, unsigned int address, unsigned int size, unsigned int compression, dc_event_progress_t *progress)
 {
 	dc_device_t *abstract = (dc_device_t *) device;
 	dc_status_t rc = DC_STATUS_SUCCESS;
@@ -371,9 +371,11 @@ shearwater_common_download (shearwater_common_device_t *device, dc_buffer_t *buf
 	}
 
 	// Enable progress notifications.
-	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
-	progress.maximum = 3 + size + 1;
-	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
+	unsigned int initial = 0, current = 0, maximum = 3 + size + 1;
+	if (progress) {
+		initial = progress->current;
+		device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
+	}
 
 	// Transfer the init request.
 	rc = shearwater_common_transfer (device, req_init, sizeof (req_init), response, 3, &n);
@@ -388,8 +390,11 @@ shearwater_common_download (shearwater_common_device_t *device, dc_buffer_t *buf
 	}
 
 	// Update and emit a progress event.
-	progress.current += 3;
-	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
+	if (progress) {
+		current += 3;
+		progress->current = initial + STEP (current, maximum);
+		device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
+	}
 
 	unsigned int done = 0;
 	unsigned char block = 1;
@@ -416,8 +421,11 @@ shearwater_common_download (shearwater_common_device_t *device, dc_buffer_t *buf
 		}
 
 		// Update and emit a progress event.
-		progress.current += length;
-		device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
+		if (progress) {
+			current += length;
+			progress->current = initial + STEP (current, maximum);
+			device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
+		}
 
 		if (compression) {
 			if (shearwater_common_decompress_lre (response + 2, length, buffer, &done) != 0) {
@@ -455,8 +463,11 @@ shearwater_common_download (shearwater_common_device_t *device, dc_buffer_t *buf
 	}
 
 	// Update and emit a progress event.
-	progress.current += 1;
-	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
+	if (progress) {
+		current += 1;
+		progress->current = initial + STEP (current, maximum);
+		device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
+	}
 
 	return DC_STATUS_SUCCESS;
 }
