@@ -31,18 +31,20 @@ extern "C" {
 
 #define ID_SERIAL   0x8010
 #define ID_FIRMWARE 0x8011
-#define ID_HARDWARE_TYPE 0x8050
+#define ID_HARDWARE 0x8050
 
 #define PREDATOR 2
 #define PETREL   3
-#define PETREL2  4
-#define NERD     5
-#define PERDIX   6
-#define PERDIXAI 7
+#define NERD     4
+#define PERDIX   5
+#define PERDIXAI 6
+
+#define NSTEPS    10000
+#define STEP(i,n) ((NSTEPS * (i) + (n) / 2) / (n))
 
 typedef struct shearwater_common_device_t {
 	dc_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
 } shearwater_common_device_t;
 
 dc_status_t
@@ -55,7 +57,7 @@ dc_status_t
 shearwater_common_transfer (shearwater_common_device_t *device, const unsigned char input[], unsigned int isize, unsigned char output[], unsigned int osize, unsigned int *actual);
 
 dc_status_t
-shearwater_common_download (shearwater_common_device_t *device, dc_buffer_t *buffer, unsigned int address, unsigned int size, unsigned int compression);
+shearwater_common_download (shearwater_common_device_t *device, dc_buffer_t *buffer, unsigned int address, unsigned int size, unsigned int compression, dc_event_progress_t *progress);
 
 dc_status_t
 shearwater_common_identifier (shearwater_common_device_t *device, dc_buffer_t *buffer, unsigned int id);
