@@ -23,10 +23,9 @@
 #include <stdlib.h> // malloc, free
 #include <assert.h> // assert
 
-#include <libdivecomputer/suunto_d9.h>
-
-#include "context-private.h"
+#include "suunto_d9.h"
 #include "suunto_common2.h"
+#include "context-private.h"
 #include "serial.h"
 #include "checksum.h"
 #include "array.h"
@@ -41,10 +40,11 @@
 #define DX       0x1C
 #define VYPERNOVO 0x1D
 #define ZOOPNOVO  0x1E
+#define D4F       0x20
 
 typedef struct suunto_d9_device_t {
 	suunto_common2_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
 } suunto_d9_device_t;
 
 static dc_status_t suunto_d9_device_packet (dc_device_t *abstract, const unsigned char command[], unsigned int csize, unsigned char answer[], unsigned int asize, unsigned int size);
@@ -59,6 +59,7 @@ static const suunto_common2_device_vtable_t suunto_d9_device_vtable = {
 		suunto_common2_device_write, /* write */
 		suunto_common2_device_dump, /* dump */
 		suunto_common2_device_foreach, /* foreach */
+		NULL, /* timesync */
 		suunto_d9_device_close /* close */
 	},
 	suunto_d9_device_packet
@@ -101,7 +102,8 @@ suunto_d9_device_autodetect (suunto_d9_device_t *device, unsigned int model)
 	// Use the model number as a hint to speedup the detection.
 	unsigned int hint = 0;
 	if (model == D4i || model == D6i || model == D9tx ||
-		model == DX || model == VYPERNOVO || model == ZOOPNOVO)
+		model == DX || model == VYPERNOVO || model == ZOOPNOVO ||
+		model == D4F)
 		hint = 1;
 
 	for (unsigned int i = 0; i < C_ARRAY_SIZE(baudrates); ++i) {
@@ -109,7 +111,7 @@ suunto_d9_device_autodetect (suunto_d9_device_t *device, unsigned int model)
 		unsigned int idx = (hint + i) % C_ARRAY_SIZE(baudrates);
 
 		// Adjust the baudrate.
-		status = dc_serial_configure (device->port, baudrates[idx], 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
+		status = dc_iostream_configure (device->iostream, baudrates[idx], 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to set the terminal attributes.");
 			return status;
@@ -145,41 +147,41 @@ suunto_d9_device_open (dc_device_t **out, dc_context_t *context, const char *nam
 	suunto_common2_device_init (&device->base);
 
 	// Set the default values.
-	device->port = NULL;
+	device->iostream = NULL;
 
 	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
+	status = dc_serial_open (&device->iostream, context, name);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to open the serial port.");
 		goto error_free;
 	}
 
 	// Set the serial communication protocol (9600 8N1).
-	status = dc_serial_configure (device->port, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
+	status = dc_iostream_configure (device->iostream, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_close;
 	}
 
 	// Set the timeout for receiving data (3000 ms).
-	status = dc_serial_set_timeout (device->port, 3000);
+	status = dc_iostream_set_timeout (device->iostream, 3000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
 		goto error_close;
 	}
 
 	// Set the DTR line (power supply for the interface).
-	status = dc_serial_set_dtr (device->port, 1);
+	status = dc_iostream_set_dtr (device->iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the DTR line.");
 		goto error_close;
 	}
 
 	// Give the interface 100 ms to settle and draw power up.
-	dc_serial_sleep (device->port, 100);
+	dc_iostream_sleep (device->iostream, 100);
 
 	// Make sure everything is in a sane state.
-	dc_serial_purge (device->port, DC_DIRECTION_ALL);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	// Try to autodetect the protocol variant.
 	status = suunto_d9_device_autodetect (device, model);
@@ -191,7 +193,8 @@ suunto_d9_device_open (dc_device_t **out, dc_context_t *context, const char *nam
 	// Override the base class values.
 	model = device->base.version[0];
 	if (model == D4i || model == D6i || model == D9tx ||
-		model == VYPERNOVO || model == ZOOPNOVO)
+		model == VYPERNOVO || model == ZOOPNOVO ||
+		model == D4F)
 		device->base.layout = &suunto_d9tx_layout;
 	else if (model == DX)
 		device->base.layout = &suunto_dx_layout;
@@ -203,7 +206,7 @@ suunto_d9_device_open (dc_device_t **out, dc_context_t *context, const char *nam
 	return DC_STATUS_SUCCESS;
 
 error_close:
-	dc_serial_close (device->port);
+	dc_iostream_close (device->iostream);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
@@ -218,7 +221,7 @@ suunto_d9_device_close (dc_device_t *abstract)
 	dc_status_t rc = DC_STATUS_SUCCESS;
 
 	// Close the device.
-	rc = dc_serial_close (device->port);
+	rc = dc_iostream_close (device->iostream);
 	if (rc != DC_STATUS_SUCCESS) {
 		dc_status_set_error(&status, rc);
 	}
@@ -237,10 +240,10 @@ suunto_d9_device_packet (dc_device_t *abstract, const unsigned char command[], u
 		return DC_STATUS_CANCELLED;
 
 	// Clear RTS to send the command.
-	dc_serial_set_rts (device->port, 0);
+	dc_iostream_set_rts (device->iostream, 0);
 
 	// Send the command to the dive computer.
-	status = dc_serial_write (device->port, command, csize, NULL);
+	status = dc_iostream_write (device->iostream, command, csize, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -249,7 +252,7 @@ suunto_d9_device_packet (dc_device_t *abstract, const unsigned char command[], u
 	// Receive the echo.
 	unsigned char echo[128] = {0};
 	assert (sizeof (echo) >= csize);
-	status = dc_serial_read (device->port, echo, csize, NULL);
+	status = dc_iostream_read (device->iostream, echo, csize, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the echo.");
 		return status;
@@ -262,10 +265,10 @@ suunto_d9_device_packet (dc_device_t *abstract, const unsigned char command[], u
 	}
 
 	// Set RTS to receive the reply.
-	dc_serial_set_rts (device->port, 1);
+	dc_iostream_set_rts (device->iostream, 1);
 
 	// Receive the answer of the dive computer.
-	status = dc_serial_read (device->port, answer, asize, NULL);
+	status = dc_iostream_read (device->iostream, answer, asize, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
