@@ -21,9 +21,9 @@
 
 #include <stdlib.h> // malloc, free
 
-#include <libdivecomputer/suunto_solution.h>
 #include <libdivecomputer/units.h>
 
+#include "suunto_solution.h"
 #include "context-private.h"
 #include "device-private.h"
 #include "ringbuffer.h"
@@ -39,7 +39,7 @@
 
 typedef struct suunto_solution_device_t {
 	dc_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
 } suunto_solution_device_t;
 
 static dc_status_t suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer);
@@ -54,9 +54,12 @@ static const dc_device_vtable_t suunto_solution_device_vtable = {
 	NULL, /* write */
 	suunto_solution_device_dump, /* dump */
 	suunto_solution_device_foreach, /* foreach */
+	NULL, /* timesync */
 	suunto_solution_device_close /* close */
 };
 
+static dc_status_t
+suunto_solution_extract_dives (dc_device_t *device, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata);
 
 dc_status_t
 suunto_solution_device_open (dc_device_t **out, dc_context_t *context, const char *name)
@@ -75,31 +78,31 @@ suunto_solution_device_open (dc_device_t **out, dc_context_t *context, const cha
 	}
 
 	// Set the default values.
-	device->port = NULL;
+	device->iostream = NULL;
 
 	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
+	status = dc_serial_open (&device->iostream, context, name);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to open the serial port.");
 		goto error_free;
 	}
 
 	// Set the serial communication protocol (1200 8N2).
-	status = dc_serial_configure (device->port, 1200, 8, DC_PARITY_NONE, DC_STOPBITS_TWO, DC_FLOWCONTROL_NONE);
+	status = dc_iostream_configure (device->iostream, 1200, 8, DC_PARITY_NONE, DC_STOPBITS_TWO, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_close;
 	}
 
 	// Set the timeout for receiving data (1000ms).
-	status = dc_serial_set_timeout (device->port, 1000);
+	status = dc_iostream_set_timeout (device->iostream, 1000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
 		goto error_close;
 	}
 
 	// Clear the RTS line.
-	status = dc_serial_set_rts (device->port, 0);
+	status = dc_iostream_set_rts (device->iostream, 0);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the DTR/RTS line.");
 		goto error_close;
@@ -110,7 +113,7 @@ suunto_solution_device_open (dc_device_t **out, dc_context_t *context, const cha
 	return DC_STATUS_SUCCESS;
 
 error_close:
-	dc_serial_close (device->port);
+	dc_iostream_close (device->iostream);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
@@ -125,7 +128,7 @@ suunto_solution_device_close (dc_device_t *abstract)
 	dc_status_t rc = DC_STATUS_SUCCESS;
 
 	// Close the device.
-	rc = dc_serial_close (device->port);
+	rc = dc_iostream_close (device->iostream);
 	if (rc != DC_STATUS_SUCCESS) {
 		dc_status_set_error(&status, rc);
 	}
@@ -158,14 +161,14 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	unsigned char answer[3] = {0};
 
 	// Assert DTR
-	dc_serial_set_dtr (device->port, 1);
+	dc_iostream_set_dtr (device->iostream, 1);
 
 	// Send: 0xFF
 	command[0] = 0xFF;
-	dc_serial_write (device->port, command, 1, NULL);
+	dc_iostream_write (device->iostream, command, 1, NULL);
 
 	// Receive: 0x3F
-	status = dc_serial_read (device->port, answer, 1, NULL);
+	status = dc_iostream_read (device->iostream, answer, 1, NULL);
 	if (status != DC_STATUS_SUCCESS)
 		return status;
 	if (answer[0] != 0x3F)
@@ -175,7 +178,7 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	command[0] = 0x4D;
 	command[1] = 0x01;
 	command[2] = 0x01;
-	dc_serial_write (device->port, command, 3, NULL);
+	dc_iostream_write (device->iostream, command, 3, NULL);
 
 	// Update and emit a progress event.
 	progress.current += 1;
@@ -184,7 +187,7 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	data[0] = 0x00;
 	for (unsigned int i = 1; i < SZ_MEMORY; ++i) {
 		// Receive: 0x01, i, data[i]
-		status = dc_serial_read (device->port, answer, 3, NULL);
+		status = dc_iostream_read (device->iostream, answer, 3, NULL);
 		if (status != DC_STATUS_SUCCESS)
 			return status;
 		if (answer[0] != 0x01 || answer[1] != i)
@@ -192,10 +195,10 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 		// Send: i
 		command[0] = i;
-		dc_serial_write (device->port, command, 1, NULL);
+		dc_iostream_write (device->iostream, command, 1, NULL);
 
 		// Receive: data[i]
-		status = dc_serial_read (device->port, data + i, 1, NULL);
+		status = dc_iostream_read (device->iostream, data + i, 1, NULL);
 		if (status != DC_STATUS_SUCCESS)
 			return status;
 		if (data[i] != answer[2])
@@ -203,7 +206,7 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 		// Send: 0x0D
 		command[0] = 0x0D;
-		dc_serial_write (device->port, command, 1, NULL);
+		dc_iostream_write (device->iostream, command, 1, NULL);
 
 		// Update and emit a progress event.
 		progress.current += 1;
@@ -211,7 +214,7 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	}
 
 	// Receive: 0x02, 0x00, 0x80
-	status = dc_serial_read (device->port, answer, 3, NULL);
+	status = dc_iostream_read (device->iostream, answer, 3, NULL);
 	if (status != DC_STATUS_SUCCESS)
 		return status;
 	if (answer[0] != 0x02 || answer[1] != 0x00 || answer[2] != 0x80)
@@ -219,10 +222,10 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 	// Send: 0x80
 	command[0] = 0x80;
-	dc_serial_write (device->port, command, 1, NULL);
+	dc_iostream_write (device->iostream, command, 1, NULL);
 
 	// Receive: 0x80
-	status = dc_serial_read (device->port, answer, 1, NULL);
+	status = dc_iostream_read (device->iostream, answer, 1, NULL);
 	if (status != DC_STATUS_SUCCESS)
 		return status;
 	if (answer[0] != 0x80)
@@ -230,10 +233,10 @@ suunto_solution_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 	// Send: 0x20
 	command[0] = 0x20;
-	dc_serial_write (device->port, command, 1, NULL);
+	dc_iostream_write (device->iostream, command, 1, NULL);
 
 	// Receive: 0x3F
-	status = dc_serial_read (device->port, answer, 1, NULL);
+	status = dc_iostream_read (device->iostream, answer, 1, NULL);
 	if (status != DC_STATUS_SUCCESS)
 		return status;
 	if (answer[0] != 0x3F)
@@ -281,7 +284,7 @@ suunto_solution_device_foreach (dc_device_t *abstract, dc_dive_callback_t callba
 }
 
 
-dc_status_t
+static dc_status_t
 suunto_solution_extract_dives (dc_device_t *abstract, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata)
 {
 	if (abstract && !ISINSTANCE (abstract))
