@@ -23,8 +23,7 @@
 #include <stdlib.h> // malloc, free
 #include <assert.h> // assert
 
-#include <libdivecomputer/uwatec_memomouse.h>
-
+#include "uwatec_memomouse.h"
 #include "context-private.h"
 #include "device-private.h"
 #include "serial.h"
@@ -40,7 +39,7 @@
 
 typedef struct uwatec_memomouse_device_t {
 	dc_device_t base;
-	dc_serial_t *port;
+	dc_iostream_t *iostream;
 	unsigned int timestamp;
 	unsigned int devtime;
 	dc_ticks_t systime;
@@ -59,9 +58,12 @@ static const dc_device_vtable_t uwatec_memomouse_device_vtable = {
 	NULL, /* write */
 	uwatec_memomouse_device_dump, /* dump */
 	uwatec_memomouse_device_foreach, /* foreach */
+	NULL, /* timesync */
 	uwatec_memomouse_device_close /* close */
 };
 
+static dc_status_t
+uwatec_memomouse_extract_dives (dc_device_t *device, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata);
 
 dc_status_t
 uwatec_memomouse_device_open (dc_device_t **out, dc_context_t *context, const char *name)
@@ -80,55 +82,55 @@ uwatec_memomouse_device_open (dc_device_t **out, dc_context_t *context, const ch
 	}
 
 	// Set the default values.
-	device->port = NULL;
+	device->iostream = NULL;
 	device->timestamp = 0;
 	device->systime = (dc_ticks_t) -1;
 	device->devtime = 0;
 
 	// Open the device.
-	status = dc_serial_open (&device->port, context, name);
+	status = dc_serial_open (&device->iostream, context, name);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to open the serial port.");
 		goto error_free;
 	}
 
 	// Set the serial communication protocol (9600 8N1).
-	status = dc_serial_configure (device->port, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
+	status = dc_iostream_configure (device->iostream, 9600, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the terminal attributes.");
 		goto error_close;
 	}
 
 	// Set the timeout for receiving data (1000 ms).
-	status = dc_serial_set_timeout (device->port, 1000);
+	status = dc_iostream_set_timeout (device->iostream, 1000);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
 		goto error_close;
 	}
 
 	// Clear the DTR line.
-	status = dc_serial_set_dtr (device->port, 0);
+	status = dc_iostream_set_dtr (device->iostream, 0);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to clear the DTR line.");
 		goto error_close;
 	}
 
 	// Clear the RTS line.
-	status = dc_serial_set_rts (device->port, 0);
+	status = dc_iostream_set_rts (device->iostream, 0);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to clear the RTS line.");
 		goto error_close;
 	}
 
 	// Make sure everything is in a sane state.
-	dc_serial_purge (device->port, DC_DIRECTION_ALL);
+	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	*out = (dc_device_t*) device;
 
 	return DC_STATUS_SUCCESS;
 
 error_close:
-	dc_serial_close (device->port);
+	dc_iostream_close (device->iostream);
 error_free:
 	dc_device_deallocate ((dc_device_t *) device);
 	return status;
@@ -143,7 +145,7 @@ uwatec_memomouse_device_close (dc_device_t *abstract)
 	dc_status_t rc = DC_STATUS_SUCCESS;
 
 	// Close the device.
-	rc = dc_serial_close (device->port);
+	rc = dc_iostream_close (device->iostream);
 	if (rc != DC_STATUS_SUCCESS) {
 		dc_status_set_error(&status, rc);
 	}
@@ -178,7 +180,7 @@ uwatec_memomouse_read_packet (uwatec_memomouse_device_t *device, unsigned char d
 	assert (result != NULL);
 
 	// Receive the header of the package.
-	status = dc_serial_read (device->port, data, 1, NULL);
+	status = dc_iostream_read (device->iostream, data, 1, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -195,7 +197,7 @@ uwatec_memomouse_read_packet (uwatec_memomouse_device_t *device, unsigned char d
 	}
 
 	// Receive the remaining part of the package.
-	status = dc_serial_read (device->port, data + 1, len + 1, NULL);
+	status = dc_iostream_read (device->iostream, data + 1, len + 1, NULL);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -232,11 +234,11 @@ uwatec_memomouse_read_packet_outer (uwatec_memomouse_device_t *device, unsigned
 			return rc;
 
 		// Flush the input buffer.
-		dc_serial_purge (device->port, DC_DIRECTION_INPUT);
+		dc_iostream_purge (device->iostream, DC_DIRECTION_INPUT);
 
 		// Reject the packet.
 		unsigned char value = NAK;
-		status = dc_serial_write (device->port, &value, 1, NULL);
+		status = dc_iostream_write (device->iostream, &value, 1, NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to reject the packet.");
 			return status;
@@ -275,7 +277,7 @@ uwatec_memomouse_read_packet_inner (uwatec_memomouse_device_t *device, dc_buffer
 
 		// Accept the packet.
 		unsigned char value = ACK;
-		status = dc_serial_write (device->port, &value, 1, NULL);
+		status = dc_iostream_write (device->iostream, &value, 1, NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to accept the packet.");
 			return status;
@@ -342,22 +344,22 @@ uwatec_memomouse_dump_internal (uwatec_memomouse_device_t *device, dc_buffer_t *
 	device_event_emit (&device->base, DC_EVENT_PROGRESS, &progress);
 
 	// Waiting for greeting message.
-	while (dc_serial_get_available (device->port, &available) == DC_STATUS_SUCCESS && available == 0) {
+	while (dc_iostream_get_available (device->iostream, &available) == DC_STATUS_SUCCESS && available == 0) {
 		if (device_is_cancelled (abstract))
 			return DC_STATUS_CANCELLED;
 
 		// Flush the input buffer.
-		dc_serial_purge (device->port, DC_DIRECTION_INPUT);
+		dc_iostream_purge (device->iostream, DC_DIRECTION_INPUT);
 
 		// Reject the packet.
 		unsigned char value = NAK;
-		status = dc_serial_write (device->port, &value, 1, NULL);
+		status = dc_iostream_write (device->iostream, &value, 1, NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to reject the packet.");
 			return status;
 		}
 
-		dc_serial_sleep (device->port, 300);
+		dc_iostream_sleep (device->iostream, 300);
 	}
 
 	// Read the ID string.
@@ -380,24 +382,24 @@ uwatec_memomouse_dump_internal (uwatec_memomouse_device_t *device, dc_buffer_t *
 
 	// Wait a small amount of time before sending the command.
 	// Without this delay, the transfer will fail most of the time.
-	dc_serial_sleep (device->port, 50);
+	dc_iostream_sleep (device->iostream, 50);
 
 	// Keep send the command to the device,
 	// until the ACK answer is received.
 	unsigned char answer = NAK;
 	while (answer == NAK) {
 		// Flush the input buffer.
-		dc_serial_purge (device->port, DC_DIRECTION_INPUT);
+		dc_iostream_purge (device->iostream, DC_DIRECTION_INPUT);
 
 		// Send the command to the device.
-		status = dc_serial_write (device->port, command, sizeof (command), NULL);
+		status = dc_iostream_write (device->iostream, command, sizeof (command), NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to send the command.");
 			return status;
 		}
 
 		// Wait for the answer (ACK).
-		status = dc_serial_read (device->port, &answer, 1, NULL);
+		status = dc_iostream_read (device->iostream, &answer, 1, NULL);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the answer.");
 			return status;
@@ -411,12 +413,12 @@ uwatec_memomouse_dump_internal (uwatec_memomouse_device_t *device, dc_buffer_t *
 	}
 
 	// Wait for the data packet.
-	while (dc_serial_get_available (device->port, &available) == DC_STATUS_SUCCESS && available == 0) {
+	while (dc_iostream_get_available (device->iostream, &available) == DC_STATUS_SUCCESS && available == 0) {
 		if (device_is_cancelled (abstract))
 			return DC_STATUS_CANCELLED;
 
 		device_event_emit (&device->base, DC_EVENT_WAITING, NULL);
-		dc_serial_sleep (device->port, 100);
+		dc_iostream_sleep (device->iostream, 100);
 	}
 
 	// Fetch the current system time.
@@ -456,10 +458,10 @@ uwatec_memomouse_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 	// Give the interface some time to notice the DTR
 	// line change from a previous transfer (if any).
-	dc_serial_sleep (device->port, 500);
+	dc_iostream_sleep (device->iostream, 500);
 
 	// Set the DTR line.
-	rc = dc_serial_set_dtr (device->port, 1);
+	rc = dc_iostream_set_dtr (device->iostream, 1);
 	if (rc != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to set the RTS line.");
 		return rc;
@@ -469,7 +471,7 @@ uwatec_memomouse_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 	status = uwatec_memomouse_dump_internal (device, buffer);
 
 	// Clear the DTR line again.
-	rc = dc_serial_set_dtr (device->port, 0);
+	rc = dc_iostream_set_dtr (device->iostream, 0);
 	if (rc != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to set the RTS line.");
 		return rc;
@@ -501,7 +503,7 @@ uwatec_memomouse_device_foreach (dc_device_t *abstract, dc_dive_callback_t callb
 }
 
 
-dc_status_t
+static dc_status_t
 uwatec_memomouse_extract_dives (dc_device_t *abstract, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata)
 {
 	if (abstract && !ISINSTANCE (abstract))
